/* Experimenteller Webserver 0.2 (c) 2003 by Dennis Schwerdel
 * Dieses Programm ist freie Software, es kann nur im Rahmen der
 * GNU General Public License (GPL) Version 2 oder neuer verwendet werden.
 * -> Siehe license.txt
 */

package config.xml ;

import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import java.io.* ;

/**
 * @author Dennis Schwerdel
 */
public class XMLDocument extends XMLNode {

  public XMLDocument () {
    super ( null ) ;
  }

  /**
   * oeffnet eine XML-Datei
   */
  public XMLDocument ( String file ) throws XMLException {
    loadFromFile ( file ) ;
  }

  /**
   * oeffnet eine XML-Datei
   * @param file Dateiname
   */
  public void loadFromFile ( String file ) throws XMLException {
    try {
      DocumentBuilderFactory builder = DocumentBuilderFactory.newInstance () ;
      builder.setValidating ( false ) ;
      Document d = builder.newDocumentBuilder().parse ( file ) ;
      super.doc = d ;
      super.node = d ;
    } catch ( Exception e ) {
      throw new XMLException ( "Cannot parse " + file + ", " + e.getMessage () ) ;
    }
  }

  /**
   * speichert eine Datei
   * @param file Dateiname
   */
  public void saveToFile ( String file ) throws IOException {
    FileWriter fw = new FileWriter ( file ) ;
    fw.write ( toString () ) ;
    fw.close () ;
  }

  /**
   * liefert eine formatierte XML-Ausgabe mit xml-version-header
   * @see XMLNode#toString()
   */
  public String toString () {
    String middle = "<?xml version='1.0' encoding='utf-8'?>\n" ;
    NodeList subnodes = node.getChildNodes() ;
    for ( int i = 0; i < subnodes.getLength(); i++ ) switch ( subnodes.item(i).getNodeType() ) {
      case Node.ELEMENT_NODE : middle += "\n" + new XMLNode ( doc, subnodes.item(i) ).toString ( "" ) ; break ;
      case Node.TEXT_NODE : middle = subnodes.item(i).getNodeValue() + middle ;
    }
    return middle + "\n" ;
  }

}
