/* Experimenteller Webserver 0.2 (c) 2003 by Dennis Schwerdel
 * Dieses Programm ist freie Software, es kann nur im Rahmen der
 * GNU General Public License (GPL) Version 2 oder neuer verwendet werden.
 * -> Siehe license.txt
 */

package config.xml ;

import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;
import java.util.StringTokenizer;

/**
 * @author Dennis Schwerdel
 */
public class XMLNode {

  protected Node node ;

  protected Document doc ;

  /**
   * erzeugt ein Node-Object mit vorgegeben Typ und Daten
   */
  protected Node createNode ( short type, String data ) throws XMLException {
    try {
      switch ( type ) {
        case Node.ATTRIBUTE_NODE : return doc.createAttribute ( data ) ;
        case Node.ELEMENT_NODE : return doc.createElement ( data ) ;
        case Node.TEXT_NODE : return doc.createTextNode ( data ) ;
        default : return doc.createElement ( data ) ;
      }
    } catch ( Exception e ) {
      throw new XMLException ( "Cannot create node: " + e.getMessage () ) ;
    }
  }

  /**
   * leerer Konstruktor
   */
  protected XMLNode () {
  }

  /**
   * Erzeugt einen Knoten im gegebenen Dokument
   */
  protected XMLNode ( Document doc, String name ) { // erzeugt einen leeren Knoten
    try {
      if ( doc == null ) doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument() ;
      this.doc = doc ;
      this.node = createNode ( Node.ELEMENT_NODE, name ) ;
    } catch ( Exception e ) {
      System.err.println ( "Impossible Exception 1: javax.xml.parsers.ParserConfigurationException in XMLNode" ) ;
    }
  }

  /**
   * Erzeugt einen Knoten im gegebenen Dokument
   */
  protected XMLNode ( Document doc, Node node ) { // erzeugt einen Knoten
    try {
      if ( doc == null ) doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument() ;
      this.doc = doc ;
      this.node = node ;
    } catch ( Exception e ) {
      System.err.println ( "Impossible Exception 2: javax.xml.parsers.ParserConfigurationException in XMLNode" ) ;
    }
  }

  /**
   * erzeugt einen Knoten
   */
  protected XMLNode ( Document doc ) {
    try {
      if ( doc == null ) doc = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument() ;
      this.doc = doc ;
      this.node = doc ;
    } catch ( Exception e ) {
      System.err.println ( "Impossible Exception 3: javax.xml.parsers.ParserConfigurationException in XMLNode" ) ;
    }
  }

  private static int find ( String hay, String needle ) {
    int t = hay.lastIndexOf ( needle ) ;
    return t < 0 ? hay.length() : t ;
  }

  /**
   * sucht ein Objekt und erzeugt dieses evtl.
   */
  protected XMLNode getNode ( String path, boolean create ) throws XMLException {
    int t = Math.min ( find ( path, "#" ), find ( path, "[]" ) ) ;
    String realPath = path.substring ( 0, t ) ;
    if ( realPath.equals("") ) return this ;
    StringTokenizer st = new StringTokenizer ( realPath, "." ) ;
    Node n = this.node ;
    while ( st.hasMoreTokens() ) {
      String s = st.nextToken() ;
      String name = s.substring ( 0, find ( s, "[" ) ) ;
      String numString = s.substring ( Math.min ( find ( s, "[" ) + 1, find ( s, "#" ) ) ) ;
      if ( ! numString.equals ( "" ) ) numString = numString.substring ( 0, find ( numString, "]" ) ) ;
      int num = Math.max ( 1, Integer.parseInt ( "0" + numString ) ) ;
      NodeList nl = n.getChildNodes() ;
      int i = 0 ;
      int numc = 0 ;
      for ( i = 0; i < nl.getLength() && numc < num; i++ ) {
        if ( nl.item ( i ).getNodeName().equals ( name ) ) numc++ ;
	if ( numc >= num ) break ;
      }
      if ( numc < num ) {
        if ( ! create ) throw new XMLException ( "Node index "+num+" out of range [1.."+numc+"]" ) ;
	Node newN = null ;
	for ( int j = 0; j < num - numc; j++ ) {
	  newN = createNode ( Node.ELEMENT_NODE, name ) ;
	  try {
	    n.appendChild ( newN ) ;
	  } catch ( Exception e ) {
	    throw new XMLException ( "Cannot create second base node" ) ;
	  }
	}
	n = newN ;
      } else n = nl.item ( i ) ;
    }
    return new XMLNode ( doc, n ) ;
  }

  /**
   * sucht ein Objekt
   */
  public XMLNode getNode ( String path ) throws XMLException { // liefert einen Knoten
    return getNode ( path, false ) ;
  }

  /**
   * liefert ein Attribut dieses Knotens
   */
  public String getNodeAttribute ( String name ) throws XMLException {
    Node att = node.getAttributes().getNamedItem(name) ;
    if ( att == null ) return null ;
    return att.getNodeValue() ;
  }

  /**
   * setzt ein Attribut dieses Knotens
   */
  public void setNodeAttribute ( String name, String value ) throws XMLException {
    String att = null ;
    try {
      att = getNodeAttribute ( name ) ;
    } catch ( XMLException e ) {} ; // ignore "NotFound"
    if ( att == null ) node.getAttributes().setNamedItem ( createNode ( Node.ATTRIBUTE_NODE, name ) ) ;
    node.getAttributes().getNamedItem(name).setNodeValue ( value ) ;
  }

  /**
   * liefert den Text dieses Knotens
   */
  public String getNodeText () {
    NodeList nl = node.getChildNodes() ;
    String text = null ;
    for ( int i = 0; i < nl.getLength(); i++ ) if ( nl.item(i).getNodeType() == Node.TEXT_NODE ) text = nl.item(i).getNodeValue () ;
    return text ;
  }

  /**
   * setzt den Text dieses Knotens
   */
  public void setNodeText ( String s ) throws XMLException {
    try {
      if ( getNodeText () == null ) node.appendChild ( createNode ( Node.TEXT_NODE, s ) ) ;
      else {
        NodeList nl = node.getChildNodes() ;
        for ( int i = 0; i < nl.getLength(); i++ ) if ( nl.item(i).getNodeType() == Node.TEXT_NODE ) nl.item(i).setNodeValue ( s ) ;
      }
    } catch ( Exception e ) {
      throw new XMLException ( "Cannot set node text: " + e.getMessage () ) ;
    }
  }

  /**
   * setzt den Wert eines Objektes
   */
  public void setValue ( String path, String value ) throws XMLException { // setzt einen Wert
    int t = Math.min ( find ( path, "#" ), find ( path, "[]" ) ) ;
    String realPath = path.substring ( 0, t ) ;
    String appendix = path.substring ( Math.min ( t, path.length() ) ) ;
    XMLNode n = getNode ( realPath, true ) ;
    if ( appendix.equals ( "" ) ) // Node Text
     n.setNodeText ( value ) ;
    else if ( appendix.charAt ( 0 ) == '#' ) // Node Attribute
     n.setNodeAttribute ( appendix.substring ( 1 ), value ) ;
    else if ( appendix.indexOf ( "[]" ) >= 0 ) {
      throw new XMLException ( "Cannot assign a value to this path." ) ;
    } else throw new XMLException ( "Unrecognized path: " + path ) ;
  }

  /**
   * liefert den Wert eines Objektes
   */
  public String getValue ( String path ) throws XMLException { // liefert einen Wert
    int t = Math.min ( find ( path, "#" ), find ( path, "[]" ) ) ;
    String realPath = path.substring ( 0, t ) ;
    String appendix = path.substring ( Math.min ( t, path.length() ) ) ;
    XMLNode n = getNode ( realPath, false ) ;
    if ( appendix.equals ( "" ) ) // Node Text
     return n.getNodeText() ;
    else if ( appendix.charAt ( 0 ) == '#' ) // Node Attribute
     return n.getNodeAttribute ( appendix.substring ( 1 ) ) ;
    else if ( appendix.indexOf ( "[]" ) >= 0 ) {
      String name = appendix.substring ( appendix.indexOf ( "[]" ) + 2 ) ;
      int anz = 0 ;
      NodeList nl = n.node.getChildNodes() ;
      for ( int i = 0; i < nl.getLength(); i++ ) if ( nl.item ( i ).getNodeType() == Node.ELEMENT_NODE && ( name.equals ( "" ) || nl.item ( i ).getNodeName().equals ( name ) ) ) anz++ ;
      return ""+anz ;
    } else throw new XMLException ( "Unrecognized path: " + path ) ;
  }

  /**
   * liefert eine formatierte XML-Ausgabe
   */
  protected String toString ( String indent ) {
    String before = indent + "<" + node.getNodeName() ;
    String middle = "" ;
    NodeList subnodes = node.getChildNodes() ;
    for ( int i = 0; i < subnodes.getLength(); i++ ) switch ( subnodes.item(i).getNodeType() ) {
      case Node.ELEMENT_NODE : middle += "\n" + new XMLNode ( doc, subnodes.item(i) ).toString ( indent + "  " ) ; break ;
      case Node.TEXT_NODE : middle = subnodes.item(i).getNodeValue() + middle ;
    }
    NamedNodeMap attributes = node.getAttributes() ;
    for ( int i = 0; i < attributes.getLength(); i++ ) before += " " + attributes.item(i).getNodeName() + "=\"" + attributes.item(i).getNodeValue() + "\"" ;
    if ( middle.equals ( "" ) ) return before + " />" ;
     else return before + ">" + middle + "\n" + indent + "</" + node.getNodeName() + ">" ;
  }

  /**
   * liefert eine formatierte XML-Ausgabe
   */
  public String toString () {
    return toString ( "" ) ;
  }

}
