/* Experimenteller Webserver 0.2 (c) 2003 by Dennis Schwerdel
 * Dieses Programm ist freie Software, es kann nur im Rahmen der
 * GNU General Public License (GPL) Version 2 oder neuer verwendet werden.
 * -> Siehe license.txt
 */


package server ;

import java.net.ServerSocket;
import java.net.Socket;
import java.net.URL ;
import java.net.URLConnection ;
import java.io.PrintStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;

public abstract class ConnectionThread extends Thread {
  protected Socket socket ;
  protected PrintStream out ;
  protected InputStreamReader in ;

  public ConnectionThread () {
    super() ;
  }

  public ConnectionThread ( Socket socket ) {
    super() ;
    assign ( socket ) ;
  }

  public void assign ( Socket socket ) {
    this.socket = socket ;
  }

  protected void write ( String data ) {
    out.print ( data ) ;
  }

  protected void writeBuffer ( byte[] data ) throws IOException {
    out.write ( data ) ;
  }

  public void printGetFile ( URLConnection urlcon, File output, int buffersize, int from, int to ) throws Exception {
    try {
      InputStream in = urlcon.getInputStream()  ;
      int length = (int)urlcon.getContentLength() ;
      if ( to < 0 || to > length ) to = length ;
      if ( from < 0 || from > length ) from = 0 ;
      FileOutputStream fos = new FileOutputStream( output ) ;
      byte[] theData ;
      int read = 0 ;
      while ( read < from ) {
        theData = new byte[Math.min ( buffersize, from - read )] ;
        in.read ( theData ) ;
        read += buffersize ;
        fos.write ( theData ) ;
      }
      while ( read < to ) {
        theData = new byte[Math.min ( buffersize, to - read )] ;
        in.read ( theData ) ;
        read += buffersize ;
        writeBuffer ( theData ) ;
        fos.write ( theData ) ;
      }
      while ( read < length ) {
        theData = new byte[Math.min ( buffersize, length - read )] ;
        in.read ( theData ) ;
        read += buffersize ;
        fos.write ( theData ) ;
      }
      fos.close () ;
      in.close () ;
    } catch ( Exception e ) {
      output.delete() ;
      throw e ;
    }
  }  
  
  public void printFile ( File theFile, int buffersize, int from, int to ) throws Exception {
    printStream ( new FileInputStream( theFile ), buffersize, from, to ) ;
  }

  public void printStream ( InputStream theStream, int buffersize, int from, int to ) throws Exception {
    int length = theStream.available () ;
    if ( to < 0 || to > length ) to = length ;
    if ( from < 0 || from > length ) from = 0 ;
    
    if ( from > 0 ) theStream.skip ( from ) ;
    int read = from ;
    byte[] theData ;
    while ( read < length ) {
      theData = new byte[Math.min ( buffersize, length - read )] ;
      theStream.read ( theData ) ;
      read += buffersize ;
      writeBuffer ( theData ) ;
    }
    theStream.close () ;
  }
  
  protected abstract void handleConnection () throws Exception ;

  public void run () {
    //long start = System.currentTimeMillis() ;
    try {
      out = new PrintStream ( socket.getOutputStream () ) ;
      in = new InputStreamReader( socket.getInputStream () ) ;
      handleConnection () ;
    } catch ( Exception e ) { System.err.println ( e.toString() ) ; }
    try {
      socket.close () ;
    } catch ( Exception e ) { System.err.println ( e.toString() ) ; }
    //long diff = System.currentTimeMillis() - start ;
    //System.out.println ( "Time : " + diff/1000.0 ) ;
  }

}
