/* Experimenteller Webserver 0.2 (c) 2003 by Dennis Schwerdel
 * Dieses Programm ist freie Software, es kann nur im Rahmen der
 * GNU General Public License (GPL) Version 2 oder neuer verwendet werden.
 * -> Siehe license.txt
 */

package server.httpd ;

import java.util.regex.* ;
import java.io.File ;

public class Path {

  private String virtual ;
  private String real ;
  private String mirror ;
  private Pattern search ;
  private Pattern replace ;
  
  public static String deleteTrailingSlash ( String str, char slash ) {
    if ( str.charAt ( str.length() - 1 ) == slash )
     str = str.substring ( 0, str.length()-1 ) ;
    return str ;
  }
  
  public static String addTrailingSlash ( String str, char slash ) {
    return deleteTrailingSlash ( str, slash ) + slash ;
  }
  
  public Path ( String virtual, String real, String mirror ) {
    this.virtual = deleteTrailingSlash ( virtual, '/' ) ;
    this.real = addTrailingSlash ( real, File.separatorChar ) ;
    this.mirror = mirror ;
    search = Pattern.compile(virtual+".*");
    replace = Pattern.compile(virtual);
  }
  
  public String getVirtual () {
    return virtual ;
  }
  
  public String getReal () {
    return real ;
  }
  
  public String getMirror () {
    return mirror ;
  }
  
  public boolean matches ( String s ) {
    return search.matcher ( s ).matches() ;
  }
  
  public String convert ( String s ) {
    if ( ! matches ( s ) ) return null ;
    return deleteTrailingSlash ( replace.matcher ( s ).replaceFirst ( real ), File.separatorChar ) ;    
  }

  public String mirror ( String s ) {
    if ( ! matches ( s ) || mirror == null ) return null ;
    return replace.matcher ( s ).replaceFirst ( mirror ) ;
  }
    
} 
