/* Experimenteller Webserver 0.2 (c) 2003 by Dennis Schwerdel
 * Dieses Programm ist freie Software, es kann nur im Rahmen der
 * GNU General Public License (GPL) Version 2 oder neuer verwendet werden.
 * -> Siehe license.txt
 */

package server.httpd ;

import config.xml.* ;
import config.Translator ;

import java.io.File;
import java.io.FileNotFoundException ;
import java.util.Arrays;
import org.w3c.dom.NodeList;

public class PathHolder implements Translator {

  private Path[] paths ;
  private String[] indexes ;
  private int count ;

  public PathHolder ( XMLDocument xml ) throws XMLException {
    count = Integer.parseInt ( xml.getValue ( "settings.[]path" ) ) ;
    paths = new Path[count] ;
    for ( int i = 1; i <= paths.length; i++ ) {
      String virtual = xml.getValue ( "settings.path["+i+"].#virtual" ) ;
      String mirror = null ;
      String absolute ;
      try {
        mirror = xml.getValue ( "settings.path["+i+"].#master" ) ;
        absolute = new File ( xml.getValue ( "settings.path["+i+"]" ) ).getCanonicalPath () ;
      } catch ( Exception e ) {
        absolute = xml.getValue ( "settings.path["+i+"]" ) ;
      }
      System.out.println ( "Mounting " + absolute + " to " + virtual ) ;
      paths[i-1] = new Path ( virtual, absolute, mirror ) ;
    }
    indexes = new String [ Integer.parseInt ( xml.getValue ( "settings.[]indexfile" ) ) ] ;
    for ( int i = 1; i <= indexes.length; i++ ) indexes[i-1] = xml.getValue ( "settings.indexfile["+i+"]" ) ;
  }

  protected boolean allowed ( String path ) {
    for ( int i = count - 1; i >= 0; i-- )
     if ( paths[i].matches ( path ) ) return true ;
    return false ;
  }

  public String mirror ( String relative ) throws Exception {
    String real = null ;
    for ( int i = count - 1; i >= 0; i-- ) if ( paths[i].matches ( relative ) ) {
      real = paths[i].mirror(relative) ;
      break ;
    }
    return real ;
  }
  
  public String resolve ( String relative ) throws Exception {
    String real = null ;
    for ( int i = count - 1; i >= 0; i-- ) if ( paths[i].matches ( relative ) ) {
      real = paths[i].convert(relative) ;
      break ;
    }
    if ( real == null ) throw new FileNotFoundException ( "Permission denied : " + relative ) ;
    if ( real.indexOf ( ".." ) >= 0 ) throw new FileNotFoundException ( "Permission denied : " + relative ) ;
    if ( new File(real).isDirectory() || real.charAt ( real.length()-1 ) == File.separatorChar ) {
     String path = Path.addTrailingSlash ( real, File.separatorChar ) ;
     for ( int i = indexes.length - 1; i >= 0; i-- )
      if ( new File ( path + indexes[i] ).exists () ) {
        real = path + indexes[i] ;
        break ;
      }
    }  
    return real ;
  }

  protected boolean subdir ( String dir, String sub ) {
    if ( sub.indexOf ( dir ) != 0 ) return false ;
    if ( sub.equals ( dir ) ) return false ;
    String rest = sub.substring ( dir.length() + 1 ) ;
    return rest.indexOf ( "/" ) < 0 ;
  }

  public String[] dirlist ( String dir ) throws Exception {
    File theFile = new File ( resolve ( dir ) ) ;
    String [] files = theFile.list () ;
    if ( files == null ) files = new String [0] ;
    for ( int i = 0; i < files.length; i++ ) {
      File item = new File ( theFile.getCanonicalPath() + File.separator + files[i] ) ;
      if ( item.isDirectory () ) files[i] = "-[ " + files[i] + " ]-" ;
    }
    int count = 0 ;
    for ( int i = 0; i < paths.length; i++ ) {
      if ( subdir ( dir, paths[i].getVirtual() ) ) count++ ;
    }
    String [] files2 = new String[files.length+count] ;
    for ( int i = 0; i < files.length; i++ ) files2[i] = files[i] ;
    for ( int i = 0; i < paths.length; i++ ) {
      if ( subdir ( dir, paths[i].getVirtual() ) ) {
        count-- ;
	files2[count+files.length] = "-[ " + paths[i].getVirtual().substring ( dir.length() ) + " ]-" ;
      }
    }
    Arrays.sort ( files2 ) ;
    return files2 ;
  }

}
